@@ -16,4 +16,6 @@ else
 end
 
 have_header("sys/times.h")
+have_func("rb_os_allocated_objects")
+have_func("rb_gc_allocated_size")
 create_makefile("ruby_prof")
