@@ -51,13 +51,18 @@
 #include <stdio.h>
 
 #include <ruby.h>
+#ifndef RUBY_VM
 #include <node.h>
 #include <st.h>
+typedef rb_event_t rb_event_flag_t;
+#define rb_sourcefile() (node ? node->nd_file : 0)
+#define rb_sourceline() (node ? nd_line(node) : 0)
+#endif
 
 
 /* ================  Constants  =================*/
 #define INITIAL_STACK_SIZE 8
-#define PROF_VERSION "0.5.2"
+#define PROF_VERSION "0.6.0"
 
 
 /* ================  Measurement  =================*/
@@ -71,6 +76,7 @@ typedef unsigned long prof_measure_t;
 #include "measure_wall_time.h"
 #include "measure_cpu_time.h"
 #include "measure_allocations.h"
+#include "measure_memory.h"
 
 static prof_measure_t (*get_measurement)() = measure_process_time;
 static double (*convert_measurement)(prof_measure_t) = convert_process_time;
@@ -195,7 +201,11 @@ figure_singleton_name(VALUE klass)
         /* Make sure to get the super class so that we don't
            mistakenly grab a T_ICLASS which would lead to
            unknown method errors. */
+#ifdef RCLASS_SUPER
+        VALUE super = rb_class_real(RCLASS_SUPER(klass));
+#else
         VALUE super = rb_class_real(RCLASS(klass)->super);
+#endif
         result = rb_str_new2("<Object::");
         rb_str_append(result, rb_inspect(super));
         rb_str_cat2(result, ">");
@@ -856,16 +866,14 @@ prof_method_cmp(VALUE self, VALUE other)
     prof_method_t *x = get_prof_method(self);
     prof_method_t *y = get_prof_method(other);
 
-    if (x->called == 0)
+    if (x->called == 0 && y->called == 0)
+      return INT2FIX(0);
+    else if (x->called == 0)
       return INT2FIX(1);
     else if (y->called == 0)
       return INT2FIX(-1);
-    else if (x->total_time < y->total_time)
-      return INT2FIX(-1);
-    else if (x->total_time == y->total_time)
-      return INT2FIX(0);
     else
-      return INT2FIX(1);
+      return rb_dbl_cmp(x->total_time, y->total_time);
 }
 
 static int
@@ -982,27 +990,27 @@ collect_threads(st_data_t key, st_data_t value, st_data_t result)
 /* ================  Profiling    =================*/
 /* Copied from eval.c */
 static char *
-get_event_name(rb_event_t event)
+get_event_name(rb_event_flag_t event)
 {
   switch (event) {
     case RUBY_EVENT_LINE:
-	return "line";
+  return "line";
     case RUBY_EVENT_CLASS:
-	return "class";
+  return "class";
     case RUBY_EVENT_END:
-	return "end";
+  return "end";
     case RUBY_EVENT_CALL:
-	return "call";
+  return "call";
     case RUBY_EVENT_RETURN:
-	return "return";
+  return "return";
     case RUBY_EVENT_C_CALL:
-	return "c-call";
+  return "c-call";
     case RUBY_EVENT_C_RETURN:
-	return "c-return";
+  return "c-return";
     case RUBY_EVENT_RAISE:
-	return "raise";
+  return "raise";
     default:
-	return "unknown";
+  return "unknown";
     }
 }
 
@@ -1070,8 +1078,13 @@ update_result(thread_data_t* thread_data,
 }
 
 
+#ifdef RUBY_VM
 static void
-prof_event_hook(rb_event_t event, NODE *node, VALUE self, ID mid, VALUE klass)
+prof_event_hook(rb_event_flag_t event, VALUE data, VALUE self, ID mid, VALUE klass)
+#else
+static void
+prof_event_hook(rb_event_flag_t event, NODE *node, VALUE self, ID mid, VALUE klass)
+#endif
 {
     
     VALUE thread;
@@ -1079,8 +1092,17 @@ prof_event_hook(rb_event_t event, NODE *node, VALUE self, ID mid, VALUE klass)
     thread_data_t* thread_data = NULL;
     long thread_id = 0;
     prof_frame_t *frame = NULL;
-    
-    /*
+#ifdef RUBY_VM
+
+    if (event != RUBY_EVENT_C_CALL &&
+  event != RUBY_EVENT_C_RETURN) {
+  VALUE thread = rb_thread_current();
+  rb_frame_method_id_and_class(&mid, &klass);
+    }
+#endif
+    /*  This code is here for debug purposes - uncomment it out
+        when debugging to see a print out of exactly what the
+        profiler is tracing. 
     {
         st_data_t key = 0;
         static unsigned long last_thread_id = 0;
@@ -1157,8 +1179,12 @@ prof_event_hook(rb_event_t event, NODE *node, VALUE self, ID mid, VALUE klass)
          called from. */
       if (frame)
       {
+#ifdef RUBY_VM
+    frame->line = rb_sourceline();
+#else
         if (node)
           frame->line = nd_line(node);
+#endif
         break;
       }
       /* If we get here there was no frame, which means this is 
@@ -1185,8 +1211,8 @@ prof_event_hook(rb_event_t event, NODE *node, VALUE self, ID mid, VALUE klass)
         
         if (!method)
         {
-          const char* source_file = (node ? node->nd_file : 0);
-          int line = (node ? nd_line(node) : 0);
+          const char* source_file = rb_sourcefile();
+          int line = rb_sourceline();
           
           /* Line numbers are not accurate for c method calls */
             if (event == RUBY_EVENT_C_CALL)
@@ -1210,8 +1236,8 @@ prof_event_hook(rb_event_t event, NODE *node, VALUE self, ID mid, VALUE klass)
           
           if (!method)
           {
-            const char* source_file = (node ? node->nd_file : 0);
-            int line = (node ? nd_line(node) : 0);
+            const char* source_file = rb_sourcefile();
+            int line = rb_sourceline();
             
             /* Line numbers are not accurate for c method calls */
             if (event == RUBY_EVENT_C_CALL)
@@ -1232,7 +1258,7 @@ prof_event_hook(rb_event_t event, NODE *node, VALUE self, ID mid, VALUE klass)
         frame->start_time = now;
         frame->wait_time = 0;
         frame->child_time = 0;
-        frame->line = node ? nd_line(node) : 0;
+        frame->line = rb_sourceline();
 
         break;
     }
@@ -1347,7 +1373,8 @@ prof_result_threads(VALUE self)
    *RubyProf::PROCESS_TIME - Measure process time.  This is default.  It is implemented using the clock functions in the C Runtime library.
    *RubyProf::WALL_TIME - Measure wall time using gettimeofday on Linx and GetLocalTime on Windows
    *RubyProf::CPU_TIME - Measure time using the CPU clock counter.  This mode is only supported on Pentium or PowerPC platforms. 
-   *RubyProf::ALLOCATIONS - Measure object allocations.  This requires a patched Ruby interpreter.*/
+   *RubyProf::ALLOCATIONS - Measure object allocations.  This requires a patched Ruby interpreter.
+   *RubyProf::MEMORY - Measure memory size.  This requires a patched Ruby interpreter.*/
 static VALUE
 prof_get_measure_mode(VALUE self)
 {
@@ -1362,7 +1389,8 @@ prof_get_measure_mode(VALUE self)
    *RubyProf::PROCESS_TIME - Measure process time.  This is default.  It is implemented using the clock functions in the C Runtime library.
    *RubyProf::WALL_TIME - Measure wall time using gettimeofday on Linx and GetLocalTime on Windows
    *RubyProf::CPU_TIME - Measure time using the CPU clock counter.  This mode is only supported on Pentium or PowerPC platforms. 
-   *RubyProf::ALLOCATIONS - Measure object allocations.  This requires a patched Ruby interpreter.*/
+   *RubyProf::ALLOCATIONS - Measure object allocations.  This requires a patched Ruby interpreter.
+   *RubyProf::MEMORY - Measure memory size.  This requires a patched Ruby interpreter.*/
 static VALUE
 prof_set_measure_mode(VALUE self, VALUE val)
 {
@@ -1400,6 +1428,13 @@ prof_set_measure_mode(VALUE self, VALUE val)
         break;
       #endif
         
+      #if defined(MEASURE_MEMORY)
+      case MEASURE_MEMORY:
+        get_measurement = measure_memory;
+        convert_measurement = convert_memory;
+        break;
+      #endif
+        
       default:
         rb_raise(rb_eArgError, "invalid mode: %d", mode);
         break;
@@ -1441,10 +1476,21 @@ prof_start(VALUE self)
     last_thread_data = NULL;
     threads_tbl = threads_table_create();
     
+#ifdef RUBY_VM
+    rb_add_event_hook(prof_event_hook,
+          RUBY_EVENT_CALL | RUBY_EVENT_RETURN |
+          RUBY_EVENT_C_CALL | RUBY_EVENT_C_RETURN 
+          | RUBY_EVENT_LINE, Qnil);
+#else
     rb_add_event_hook(prof_event_hook,
           RUBY_EVENT_CALL | RUBY_EVENT_RETURN |
           RUBY_EVENT_C_CALL | RUBY_EVENT_C_RETURN 
           | RUBY_EVENT_LINE);
+#endif
+
+#if defined(MEASURE_MEMORY)
+    rb_gc_enable_stats();
+#endif
 
     return Qnil;
 }
@@ -1457,6 +1503,10 @@ prof_start(VALUE self)
 static VALUE
 prof_stop(VALUE self)
 {
+#if defined(MEASURE_MEMORY)
+    rb_gc_disable_stats();
+#endif
+
     VALUE result = Qnil;
 
     if (threads_tbl == NULL)
@@ -1515,17 +1565,28 @@ Init_ruby_prof()
     rb_define_singleton_method(mProf, "measure_mode", prof_get_measure_mode, 0);
     rb_define_singleton_method(mProf, "measure_mode=", prof_set_measure_mode, 1);
 
+    rb_define_const(mProf, "CLOCKS_PER_SEC", INT2NUM(CLOCKS_PER_SEC));
     rb_define_const(mProf, "PROCESS_TIME", INT2NUM(MEASURE_PROCESS_TIME));
     rb_define_const(mProf, "WALL_TIME", INT2NUM(MEASURE_WALL_TIME));
 
-    #if defined(MEASURE_CPU_TIME)
+    #ifndef MEASURE_CPU_TIME
+    rb_define_const(mProf, "CPU_TIME", Qnil);
+    #else
     rb_define_const(mProf, "CPU_TIME", INT2NUM(MEASURE_CPU_TIME));
     rb_define_singleton_method(mProf, "cpu_frequency", prof_get_cpu_frequency, 0); /* in measure_cpu_time.h */
     rb_define_singleton_method(mProf, "cpu_frequency=", prof_set_cpu_frequency, 1); /* in measure_cpu_time.h */
     #endif
         
-    #if defined(MEASURE_ALLOCATIONS)
-    rb_define_const(mProf, "ALLOCATED_OBJECTS", INT2NUM(MEASURE_ALLOCATIONS));
+    #ifndef MEASURE_ALLOCATIONS
+    rb_define_const(mProf, "ALLOCATIONS", Qnil);
+    #else
+    rb_define_const(mProf, "ALLOCATIONS", INT2NUM(MEASURE_ALLOCATIONS));
+    #endif
+    
+    #ifndef MEASURE_MEMORY
+    rb_define_const(mProf, "MEMORY", Qnil);
+    #else
+    rb_define_const(mProf, "MEMORY", INT2NUM(MEASURE_MEMORY));
     #endif
     
     cResult = rb_define_class_under(mProf, "Result", rb_cObject);
