@@ -15,7 +15,8 @@ def check_parent_times(method)
     sum + call_info.self_time
   end
 
-  assert_in_delta(method.self_time, parents_self_time, 0.01, method.full_name) 
+  assert_in_delta(method.self_time, parents_self_time, 0.01, 
+                  "Invalid parent times for method #{method.full_name}")
     
   parents_wait_time = method.parents.inject(0) do |sum, call_info|
     sum + call_info.wait_time
@@ -26,7 +27,9 @@ def check_parent_times(method)
   parents_children_time = method.parents.inject(0) do |sum, call_info|
     sum + call_info.children_time
   end
-  assert_in_delta(method.children_time, parents_children_time, 0.01, method.full_name)
+  
+  assert_in_delta(method.children_time, parents_children_time, 0.01,
+                  "Invalid child times for method #{method.full_name}")
 end
   
 def check_parent_calls(method)
@@ -35,7 +38,9 @@ def check_parent_calls(method)
   parent_calls = method.parents.inject(0) do |sum, call_info|
     sum + call_info.called
   end
-  assert_equal(method.called, parent_calls, method.full_name)  
+  
+  assert_equal(method.called, parent_calls,  
+                  "Invalid parent calls for method #{method.full_name}")
 end
   
 def check_child_times(method)
@@ -45,5 +50,6 @@ def check_child_times(method)
     sum + call_info.total_time
   end
   
-  assert_in_delta(method.children_time, children_total_time, 0.01, method.full_name)
+  assert_in_delta(method.children_time, children_total_time, 0.01,
+                  "Invalid child time for method #{method.full_name}")
 end
