@@ -120,6 +120,9 @@ class ThreadTest < Test::Unit::TestCase
       end
     end
     
+    printer = RubyProf::GraphPrinter.new(result)
+    printer.print
+    
     result.threads.each do |thread_id, methods|
       STDOUT << "thread: " << thread_id << "\n"
       methods.each do |method|
